<?php
namespace Http\Message;
/**
 * Cookie Jar holds a set of Cookies.
 *
 * @author Márk Sági-Kazár <mark.sagikazar@gmail.com>
 */

final class CookieJar implements \Countable, \IteratorAggregate
{
    /**
     * @var \SplObjectStorage<Cookie, mixed>
     */
    private $cookies;
    public function __construct()
    {
        $this->cookies = new \SplObjectStorage();
    }
    /**
     * Checks if there is a cookie.
     *
     * @return bool
     */
    public function hasCookie(Cookie $cookie)
    {
        return $this->cookies->contains($cookie);
    }
    public function addCookie(Cookie $cookie)
    {
        if (!$this->hasCookie($cookie)) {
            $cookies = $this->getMatchingCookies($cookie);
            foreach ($cookies as $matchingCookie) {
                if ($cookie->getValue() !== $matchingCookie->getValue() || $cookie->getMaxAge() > $matchingCookie->getMaxAge()) {
                    $this->removeCookie($matchingCookie);
                    continue;
                }
            }
            if ($cookie->hasValue()) {
                $this->cookies->attach($cookie);
            }
        }
    }
    public function removeCookie(Cookie $cookie)
    {
        $this->cookies->detach($cookie);
    }
    public function getCookies()
    {
        $match = function ($matchCookie) {
            return true;
        };
        return $this->findMatchingCookies($match);
    }
    public function getMatchingCookies(Cookie $cookie)
    {
        $match = function ($matchCookie) use ($cookie) {
            return $matchCookie->match($cookie);
        };
        return $this->findMatchingCookies($match);
    }
    private function findMatchingCookies(callable $match)
    {
        $cookies = [];
        foreach ($this->cookies as $cookie) {
            if ($match($cookie)) {
                $cookies[] = $cookie;
            }
        }
        return $cookies;
    }
    public function hasCookies()
    {
        return $this->cookies->count() > 0;
    }
    public function setCookies(array $cookies)
    {
        $this->clear();
        $this->addCookies($cookies);
    }
    public function addCookies(array $cookies)
    {
        foreach ($cookies as $cookie) {
            $this->addCookie($cookie);
        }
    }
    public function removeCookies(array $cookies)
    {
        foreach ($cookies as $cookie) {
            $this->removeCookie($cookie);
        }
    }
    public function removeMatchingCookies($name = null, $domain = null, $path = null)
    {
        $match = function ($cookie) use ($name, $domain, $path) {
            $match = true;
            if (isset($name)) {
                $match = $match && ($cookie->getName() === $name);
            }
            if (isset($domain)) {
                $match = $match && $cookie->matchDomain($domain);
            }
            if (isset($path)) {
                $match = $match && $cookie->matchPath($path);
            }
            return $match;
        };
        $cookies = $this->findMatchingCookies($match);
        $this->removeCookies($cookies);
    }
    public function clear()
    {
        $this->cookies = new \SplObjectStorage();
    }
    #[\ReturnTypeWillChange]
    public function count()
    {
        return $this->cookies->count();
    }
    #[\ReturnTypeWillChange]
    public function getIterator()
    {
        return clone $this->cookies;
    }
}
